// C Source File
// Created 09/07/2003; 20:16:14

#include "const.h"
#include "3Dengine.h"


/************************ Dfinition missile laser ****************************/
Vertex missVertices[2] = {{0,0,15},{0,0,-15}};
BPoly missBPolys[1] ={{2, {0,1,0,0,0,0,0,0,0,0}, 1, 0}};
#define MISS_VERT_NB 2
#define MISS_POLY_NB 1
/******************************************************************************/


// Lance un missile dans le missile set indiqu
// comme partant du point startP du vaisseau dans sa position repre par shipC->pos, avec des angles shipC->ang
short launch_missile(MissSet* miss, SolidC* shipC, Vertex* startP, short speed, short backView)
{
	if (miss->missNb >= MAX_MISS) return(0); // Il n'est pas possible de lancer un autre missile
	short	index = miss->missNb; // Index auquel on va placer le nouveau missile

	Vertex startPCpy;
	startPCpy.x = startP->x;
	startPCpy.y = startP->y;
	startPCpy.z = startP->z;
	rotations_rpy(&startPCpy, &(shipC->ang)); // Positionner ce point du vaisseau dans le repre centr sur la vaisseau, donc en tenant compte des angles vaisseau

	// Enregistrement de la position de dpart
	miss->dir[index].x = 0;
	miss->dir[index].y = 0;
	miss->dir[index].z = -(((backView ? -1 : 1) * speed) + MISS_SPEED);
	rotations_py(&(miss->dir[index]), &(shipC->ang)); // Faire tourner ce vecteur directeur avec les angles du vaisseau

	miss->coord[index].pos.x = shipC->pos.x + startPCpy.x;
	miss->coord[index].pos.y = shipC->pos.y + startPCpy.y;
	miss->coord[index].pos.z = shipC->pos.z + startPCpy.z; // Position de dpart du missile ( l'endroit indiqu par startP sur le vaisseau)

	miss->coord[index].ang.x = shipC->ang.x;
	miss->coord[index].ang.y = shipC->ang.y;
	miss->coord[index].ang.z = 0; // Angles de dpart du missile

	miss->dist[index] = 0; // Le missile n'a pas encore parcouru de distance

	miss->missNb++; // Un missile de plus dans le MissSet
	return(1); // Missile lanc avec succs
}


// Effacer le missile  la position pos du MissSet miss
void delete_missile(MissSet* miss, short pos)
{
	miss->missNb--;
	memmove(&(miss->coord[pos]), &(miss->coord[pos+1]), sizeof(SolidC)*(miss->missNb-pos));
	memmove(&(miss->dir[pos]),   &(miss->dir[pos+1]),   sizeof(Vertex)*(miss->missNb-pos));
	memmove(&(miss->dist[pos]),  &(miss->dist[pos+1]),  sizeof(short) *(miss->missNb-pos)); // Dcaler vers la gauche les donnes des diffrents tableaux
}


// Dplacer les missiles d'un cran (oprations en interne dans le missile set)
void move_missiles(MissSet* miss)
{
	short i;

	for (i=0; i<miss->missNb; i++)
		{
		miss->coord[i].pos.x += miss->dir[i].x;
		miss->coord[i].pos.y += miss->dir[i].y;
		miss->coord[i].pos.z += miss->dir[i].z; // Faire avancer le missile d'un pas grce  son vecteur directeur
		if ((++miss->dist[i]) == MISS_RANGE)
			{
			delete_missile(miss, 0); // Si le missile  parcouru la distance limite, le detruire
			i--; // Pour que la boucle suivante fasse avancer le missile suivant qui est revenu  l'index i
			}
		}
}


void draw_missiles(MissSet* miss)
{
	short i;

	for (i=0; i<miss->missNb; i++)
		render_object(missVertices, MISS_VERT_NB, missBPolys, NULL, MISS_POLY_NB, &(miss->coord[i]), WHITE_LINES);
}


// Calcule de rayon de la sphre englobante des extrmits d'un vaisseau (utilis pour acclrer la collision missiles/vaisseaux)
long calc_radius(SHIP_INFO* shipNfo)
{
	short i;
	long radius = 0, pot_radius; // Rayon le plus grand trouv, et rayon potentiel (intermdiaire de calcul)

	for (i=0; i<8; i++)
		{
		pot_radius =  ((long)shipNfo->ptrVert[shipNfo->ptrEnds[i]].x) * shipNfo->ptrVert[shipNfo->ptrEnds[i]].x
								+ ((long)shipNfo->ptrVert[shipNfo->ptrEnds[i]].y) * shipNfo->ptrVert[shipNfo->ptrEnds[i]].y
								+ ((long)shipNfo->ptrVert[shipNfo->ptrEnds[i]].z) * shipNfo->ptrVert[shipNfo->ptrEnds[i]].z;
		if (pot_radius > radius) radius = pot_radius;
		}
	return (radius);
}

// Contrle la collision missiles/vaisseau et met  jour l'octet damage du vaisseau
short col_miss_ship(SHIP_INFO* shipNfo, SolidC* shipC, long sqRadius, MissSet* miss) // (sqRadius = rayon au carr)
{
	short i,j;
	long x,y,z, xe,ye,ze;

	for (i=0; i<miss->missNb; i++)
		{
		x = miss->coord[i].pos.x - shipC->pos.x;
		y = miss->coord[i].pos.y - shipC->pos.y;
		z = miss->coord[i].pos.z - shipC->pos.z; // Position du missile dans le repre du vaisseau

		if (x*x + y*y + z*z <= (sqRadius<<1)) // seulement si le missile est  l'intrieur de la sphre englobante des extrmits (rayon mutipli par racine(2)
			for (j=0; j<8; j++)
				{
				xe = shipNfo->ptrVert[shipNfo->ptrEnds[j]].x;
				ye = shipNfo->ptrVert[shipNfo->ptrEnds[j]].y;
				ze = shipNfo->ptrVert[shipNfo->ptrEnds[j]].z; // Position de l'xtrmit
				if ((x-xe)*(x-xe) + (y-ye)*(y-ye) + (z-ze)*(z-ze) <= (sqRadius>>1) && !(shipC->damage & (0x01<<j))) // Test de l'appartenance du missile  une sphre de centre l'extrmit et de rayon racine(2) fois plus petit que le prcdent (au carr -> 2)
					{ // Si collision et secteur non dja dtruit
					shipC->damage |= (0x01<<j); // Modification de l'tat du bit damage correspondant  cette extrmit
					delete_missile(miss, i); // Effacer le missile qui vient d'endommager une partie du vaisseau sur les 8
					break; // Passer au missile suivant
					}
				}
		}

	return (shipC->damage == 0xff ? TRUE : FALSE); // Retourne TRUE si le vaisseau est entirement dtruit
}


// Explosions
void start_explosion(SolidC* shipC, Explosion* expl) // Initialise l'explosion (position des bouts de tle)
{
	short i;
	Vector dirTable[6] = {{20,0,0},{-20,0,0},{0,20,0},{0,-20,0},{0,0,20},{0,0,-20}};

	// Calcul du vecteur vitesse du vaisseau
	Vector speedV;
	speedV.x = 0;
	speedV.y = 0;
	speedV.z = -shipC->speed;	// Vitesse vers l'avant
	rotations_py(&speedV, &(shipC->ang)); // Donner la bonne direction au vecteur vitesse, celle du vaisseau

	for (i=0; i<6; i++)
		{
		expl->coord[i].pos.x = shipC->pos.x; // Position initiale des toles = position du vaisseau au moment de l'explosion
		expl->coord[i].pos.y = shipC->pos.y;
		expl->coord[i].pos.z = shipC->pos.z;
		expl->coord[i].ang.x = random(512); // Angle alatoire
		expl->coord[i].ang.y = random(512);
		expl->coord[i].ang.z = random(512);
		expl->dir[i].x = speedV.x + dirTable[i].x;
		expl->dir[i].y = speedV.y + dirTable[i].y;
		expl->dir[i].z = speedV.z + dirTable[i].z;
		}

	expl->active = 1; // Explosion active
}

short manage_explosion(Explosion* expl) // Fait bouger les bouts de tle
{
	short i;

	for (i=0; i<6; i++)
		{
		expl->coord[i].pos.x += expl->dir[i].x;
		expl->coord[i].pos.y += expl->dir[i].y;
		expl->coord[i].pos.z += expl->dir[i].z;
		A_INC(expl->coord[i].ang.x,8)
		A_INC(expl->coord[i].ang.y,8)
		}
	if (expl->active == EXPL_MAX) return TRUE; // Explosion termine
	else
		{
		expl->active++;
		return FALSE;
		}
}


Vertex toleVert[4] = {{35,35,0},{-35,35,0},{-35,-35,0},{35,-35,0}};
BPoly toleBPolys[1] = {{4, {0,1,2,3,0,0,0,0,0,0},1,0}};
Vector toleNormals[1] = {{0,0,256}};

void render_explosion(Explosion* expl) // Affiche les bouts de tle
{
	short i;
	for (i=0; i<6; i++) render_object(toleVert, 4, toleBPolys, toleNormals, 1, &(expl->coord[i]), NO_OPTION);
}
