// C Source File
// Created 01/07/2003; 20:02:17

#include "const.h"
#include "3Dengine.h"

extern void DrawVirtual(void);
extern unsigned long fastsqrt(unsigned long); // Merci  Thomas Nussbaumer de la TICT (http://tict.ticalg.org/) pour la routine


// Positionne la camra derrire le vaisseau (modifie les paramtres de la camera cam)
void calc_cam_pos(Camera* cam, SolidC* ship, short backView)
{
	// La camera aura en permanence les mmes angles en pitch et en roll que le vaisseau (sauf si backView est vraie, dans ce cas le yaw est modifi)
	cam->ang.x = ((backView ? -1 : 1) * ship->ang.x) & 0x01ff;
	cam->ang.y = (ship->ang.y + (backView ? 256 : 0)) & 0x01ff;
	cam->ang.z = ((backView ? -1 : 1) * ship->ang.z) & 0x01ff;

	cam->pos.x = ship->pos.x;
	cam->pos.y = ship->pos.y;
	cam->pos.z = ship->pos.z;
}


// Commande directe du vaisseau point par ship
void direct_cmd_shp(SolidC* ship)
{
	Vector vector;

	short diams = (_keytest(RR_DIAMOND) ? 160 : 40);

	vector.x = 0;
	vector.y = 0;
	vector.z = -diams;
	rotations_py(&vector, &(ship->ang));

	if (_keytest(RR_F4)) // Avancer
		{
		ship->pos.x+=vector.x;
		ship->pos.y+=vector.y;
		ship->pos.z+=vector.z;
		}
	if (_keytest(RR_F5)) // Reculer
		{
		ship->pos.x-=vector.x;
		ship->pos.y-=vector.y;
		ship->pos.z-=vector.z;
		}

	if (!_keytest(RR_MODE)) // Faire tourner la camra autour du vaisseau (comme dans un mode replay)
		{
		if (_keytest(RR_RIGHT)) A_INC(ship->ang.y,8) // yaw
		if (_keytest(RR_LEFT)) A_DEC(ship->ang.y,8)
		if (_keytest(RR_DOWN)) A_INC(ship->ang.x,8) // pitch
		if (_keytest(RR_UP)) A_DEC(ship->ang.x,8)
		if (_keytest(RR_BCKSPC)) A_DEC(ship->ang.z,8) // roll
		if (_keytest(RR_CLEAR)) A_INC(ship->ang.z,8)
		}
}

// Commande directe de la camra
void direct_cmd_cam(Camera* cam)
{
	Vector vector;

	short diams = (_keytest(RR_DIAMOND) ? 160 : 40);

	vector.x = 0;
	vector.y = 0;
	vector.z = -diams;
	rotations_py(&vector, &(cam->ang));

	if (_keytest(RR_F4)) // Avancer
		{
		cam->pos.x+=vector.x;
		cam->pos.y+=vector.y;
		cam->pos.z+=vector.z;
		}
	if (_keytest(RR_F5)) // Reculer
		{
		cam->pos.x-=vector.x;
		cam->pos.y-=vector.y;
		cam->pos.z-=vector.z;
		}

	vector.x = diams;
	vector.y = 0;
	vector.z = 0;
	rotations_py(&vector, &(Angle){0, cam->ang.y, 0});

	if (_keytest(RR_F2)) // Strafer  droite
		{
		cam->pos.x+=vector.x;
		cam->pos.z+=vector.z;
		}
	if (_keytest(RR_F1)) // Strafer  gauche
		{
		cam->pos.x-=vector.x;
		cam->pos.z-=vector.z;
		}

	if (_keytest(RR_RIGHT)) A_INC(cam->ang.y,8) // yaw
	if (_keytest(RR_LEFT)) A_DEC(cam->ang.y,8)
	if (_keytest(RR_DOWN)) A_INC(cam->ang.x,8) // pitch
	if (_keytest(RR_UP)) A_DEC(cam->ang.x,8)
	if (_keytest(RR_BCKSPC)) A_DEC(cam->ang.z,8) // roll
	if (_keytest(RR_CLEAR)) A_INC(cam->ang.z,8)

	if (_keytest(RR_1)) cam->type = 1; // Utiliser la camra en camra 1re personne
	if (_keytest(RR_3)) cam->type = 3; // Utiliser la camra en camra 3me personne
}



// Commande du vaisseau (changements de trajectoire) in game
void cmd_run_shp(SolidC* ship, Status* shipStatus, short controlMode, unsigned char loop_cnt)
{
	short inverse = ((ship->ang.x > 128 && ship->ang.x < 384) ? -1 : 1);

	if (controlMode == SIMULATION)
		{
		if (_keytest(RR_LEFT)) A_DEC(ship->ang.z,8)
		if (_keytest(RR_RIGHT)) A_INC(ship->ang.z,8) // L'utilisateur commande le roll

		if (_keytest(RR_UP)) // Et le manche  balai avec up/down
			{
			A_INC(ship->ang.x, cos_f(ship->ang.z)>>5)
			A_DEC(ship->ang.y, inverse*sin_f(ship->ang.z)>>5)
			}
		if (_keytest(RR_DOWN))
			{
			A_DEC(ship->ang.x, cos_f(ship->ang.z)>>5)
			A_INC(ship->ang.y, inverse*sin_f(ship->ang.z)>>5)
			}
		// Il y a rpartition du changement de trajectoire entre le pitch et le yaw suivant la valeur du roll
		}
	else
		{
		if (_keytest(RR_DOWN)) A_DEC(ship->ang.x,8)
		if (_keytest(RR_UP)) A_INC(ship->ang.x,8)
		if (_keytest(RR_LEFT)) A_DEC(ship->ang.y,inverse<<3)
		if (_keytest(RR_RIGHT)) A_INC(ship->ang.y,inverse<<3)
		}

	Vector speedV;
	speedV.x = 0;
	speedV.y = 0;
	speedV.z = -ship->speed;	// Vitesse vers l'avant

	rotations_py(&speedV, &(ship->ang)); // Donner la bonne direction au vecteur vitesse, celle du vaisseau

	// Dplacer le vaisseau dans l'espace suivant son vecteur vitesse
	ship->pos.x += speedV.x;
	ship->pos.y += speedV.y;
	ship->pos.z += speedV.z;

	long norm2 = ((long)ship->pos.x)*ship->pos.x
							+((long)ship->pos.y)*ship->pos.y
							+((long)ship->pos.z)*ship->pos.z;
	if (norm2 >= 225000000 && ((loop_cnt & 0x07) <=3)) // 15000
		DrawStr(36, lcd_h-5, "! Leaving game area !", A_NORMAL); // Message clignotant (4 frames allum, 4 frames teint)
	if (norm2 >= 625000000) // 25000
		shipStatus->explStarted = TRUE; // Le vaisseau part trop loin : desertion -> explosion
		// explStarted est utilis comme message temporaire, l'explosion ne sera active que quand start_explosion sera appelle dans game.c
}


// Intelligence artificielle de commande de vaisseau

Vertex IAdest; // Point que vise en permanence le vaisseau command par l'IA
short IAmode;  // Mode de l'IA parmi : Fuite face  un tir, rparations, rparations + attaque ou poursuite (dfinitions ci-dessous)
enum IAmodes {ESCAPE, REPAIR, REPAIRnSHOOT, PURSUIT};
short pursuitDist; // Distance de poursuite en mode PURSUIT

void cmd_shp_ia(SolidC* IAshipC, Status* IAstatus, SolidC* target, Explosion* targetExpl, MissSet* miss, short controlMode)
{
BOOL range_ok = FALSE;
unsigned char mask;
short damageCount = 0, i, /*inverse,*/ step_y = 0, step_p = 0, step_r = 0, spe_step = 9;
long scal_y, scal_p, result = 0, t;
unsigned long norm1, norm2 = 0, result2/*, result3*/;
Vertex P, *M;
Vector v, dir, *mdir;
lgVector SM, dir2get, xProd/*, lP*/;

#ifdef IA_SIMU
Vector xV, yV, zV; // Base vaisseau
// Calcul de la base vaisseau :
	xV.x = 256; xV.y =   0; xV.z =   0;
	yV.x =   0; yV.y = 256; yV.z =   0;
	zV.x =   0; zV.y =   0; zV.z = 256; // Dfinitions de la base de l'univers
	rotations_rpy(&xV, &(IAshipC->ang));
	rotations_rpy(&yV, &(IAshipC->ang));
	rotations_rpy(&zV, &(IAshipC->ang)); // Et rotation de cette base
#endif

// choix du point de destination du vaisseau
// 1. Analyse de la situation (missiles  proximit, dgats)
	// Detection de missiles :
	for (i = -1; i<miss->missNb; i++)
		{
		if (i==-1)
			{
			M = &(target->pos); // Fuite face au vaisseau ennemi si ncssaire
			dir.x = 0;
			dir.y = 0;
			dir.z = -256;
			rotations_py(&dir, &(target->ang)); // Cration du vecteur directeur de la cible
			mdir = &dir;
			}
		else
			{
			M = &(miss->coord[i].pos); // Sinon fuite face au missiles
			mdir = &(miss->dir[i]);
			}

		SM.x = M->x - IAshipC->pos.x;
		SM.y = M->y - IAshipC->pos.y;
		SM.z = M->z - IAshipC->pos.z;
		norm2 = SM.x * SM.x + SM.y * SM.y + SM.z * SM.z; // Distance au carr entre le vaisseau S et le missile M

		if (norm2 <= (i==-1 ? 219961 : 3515625)) // 219961  = 1/4 de la porte des missiles (a vitesse de tir nulle) au carr
																						 // 3515625 = 1 fois la porte des missiles (a vitesse de tir nulle) au carr
			{
			#ifdef IA_SIMU
			// Calcul du "vecteur de dgagement" v
			if (controlMode == SIMULATION) v = yV;  // vecteur de base y de la base vaisseau
			else // produit vectoriel de mdir avec le vecteur de base x (dgagement vertical en mode ARCADE)
			#endif
				{
				v.x =  0;
				v.y =  mdir->z;
				v.z = -mdir->y;
				if (mdir->y == 0 && mdir->z == 0) v.y = -256; // Cas de produit vectoriel nul
				}

			/* Dans quel sens appliquer V au point S pour trouver un point de dgagement ?
				 Pour cela calcul de la projection P du point S sur la droite directrice du missile
				 Formule : OP = OM - <SM|mdir>/||mdir|| mdir o O est l'origine de l'espace
				 sachant que ||mdir|| = 256 */
			t = SM.x * mdir->x + SM.y * mdir->y + SM.z * mdir->z;
			P.x = M->x - ((t*mdir->x)>>16);
			P.y = M->y - ((t*mdir->y)>>16);
			P.z = M->z - ((t*mdir->z)>>16);

			// Maintenant si <SP|V> < 0, le vecteur de dgagement est dans le mauvais sens car il peut couper la trajectoire du missile
			if (((P.x - IAshipC->pos.x) * (long)v.x) + ((P.y - IAshipC->pos.y) * (long)v.y) + ((P.z - IAshipC->pos.z) * (long)v.z) <= 0)
				{
				v.x = -v.x;
				v.y = -v.y;
				v.z = -v.z;
				}

			while (abs(v.x) < 2000 && abs(v.y) < 2000 && abs(v.z) < 2000)
				{
				v.x = v.x<<1;
				v.y = v.y<<1;
				v.z = v.z<<1; // Donner une taille raisonnable mais pas trop au vecteur de dgagement
				}

			// Enfin nouvelle affectation pour IAdest
			IAdest.x = IAshipC->pos.x + v.x;
			IAdest.y = IAshipC->pos.y + v.y;
			IAdest.z = IAshipC->pos.z + v.z;
			break;
			}
		}
	// Comptage du nombre de secteurs endommags sur le vaisseau
	for (mask = 0x80; mask; mask = mask>>1) if (IAshipC->damage & mask) damageCount++;

// 2. Choix d'un mode entre les 4 modes fuite, rparation, rparation + attaque et poursuite
	if (targetExpl->active) // Cible dtruite (en train d'exploser)
		{
		IAmode = REPAIR;
		IAdest.x = 0;
		IAdest.y = 0;
		IAdest.z = 0; // Destination : le batiment central
		}
	else if (i<miss->missNb) // Donc un missile  t detect  proximit
		IAmode = ESCAPE;
	else if (IAshipC->damage && (((long)IAshipC->pos.x) * ((long)IAshipC->pos.x) + ((long)IAshipC->pos.y) * ((long)IAshipC->pos.y) + ((long)IAshipC->pos.z) * ((long)IAshipC->pos.z) < 250000) && IAshipC->speed == 0)
		{
		IAmode = REPAIRnSHOOT; // Si le vaisseau est dans la sphre d'influence de l'atelier de rparations, arrt et endommag, mode REPAIRnSHOOT ou le vaisseau s'oriente sur son ennemi pour pouvoir tirer
		IAdest.x = target->pos.x;
		IAdest.y = target->pos.y;
		IAdest.z = target->pos.z;
		}
	else if (damageCount >= 5) // Mode rparations si damage >= 5
		{
		IAmode = REPAIR;
		IAdest.x = 0;
		IAdest.y = 0;
		IAdest.z = 0; // Destination : le batiment central
		}
	else // Mode poursuite
		{
		IAmode = PURSUIT;
		pursuitDist = (MISS_SPEED * MISS_RANGE)>>1;
		IAdest.x = target->pos.x;
		IAdest.y = target->pos.y;
		IAdest.z = target->pos.z;
		}

// 3. Dplacements vers ce point :
 // 3.1. Prise de direction (asservissement des angles)
	dir2get.x = IAdest.x - IAshipC->pos.x;
	dir2get.y = IAdest.y - IAshipC->pos.y;
	dir2get.z = IAdest.z - IAshipC->pos.z; // Vecteur la position du vaisseau ennemi  la mienne

	while ((abs(dir2get.x) >= 200) || (abs(dir2get.y) >= 200) || (abs(dir2get.z) >= 200)) // Vecteur dir2get trop grand, reduction pour viter les overflow
		{
		dir2get.x = dir2get.x >> 1;
		dir2get.y = dir2get.y >> 1; // Diviser par 2 le vecteur
		dir2get.z = dir2get.z >> 1;
		}

	if (controlMode == ARCADE)
		{
		// Asservissement en yaw
		// Calcul du vecteur directeur du vaisseau ennemi, sur 1 seul axe de rotation (vecteur de norme 256)
		dir.x = sin_f(IAshipC->ang.y);
		dir.y = 0;
		dir.z = -cos_f(IAshipC->ang.y);

		xProd.y = dir.z * dir2get.x - dir.x * dir2get.z; // Produit vectoriel, utilis pour l'infirmation qu'il donne sur le sinus de l'angle
		scal_y = dir.x * dir2get.x + dir.z * dir2get.z; // Produit scalaire, si ngatif, angle entre les 2 vecteurs d'au moins 90 (le vaisseau va en + rallentir pour ngocier son virage)
		norm1 = fastsqrt((unsigned long)(dir2get.x * dir2get.x + dir2get.z * dir2get.z));

		// Rglage du pas de l'asservissement pour ne pas dpasser la position voulue

		result2 = abs(xProd.y); //((unsigned long)xProd.y) * ((unsigned long)xProd.y);
		if      (result2 >= 25 * norm1 || scal_y < 0) step_y = 8;	 // Rotation rapide car angle suprieur  8 units d'angle, le test sur le produit scalaire exclut la stabilisation en direction oppose
		else if (result2 >= 3 * norm1) step_y = 1; // Rotation lente, car angle entre 1 et 8 units d'angle
		else		step_y = 0; // Vaisseau align en yaw

		if      (xProd.y <= 0) A_INC(IAshipC->ang.y, step_y)
		else if (xProd.y > 0) A_DEC(IAshipC->ang.y, step_y)

		// Asservissement en pitch
		dir.x = 0;
		dir.y = 0;
		dir.z = -256;
		rotations_py(&dir, &(IAshipC->ang)); // Calcul du vecteur directeur du vaisseau

		xProd.x = dir.y * dir2get.z - dir.z * dir2get.y;
		xProd.z = dir.x * dir2get.y - dir.y * dir2get.x; // Produit vectoriel qui sera ensuite projett
		scal_p  = dir.x  * dir2get.x + dir.y * dir2get.y + dir.z * dir2get.z; // Produit scalaire pour viter la stabilisation en direction oppose et prvoir un rallentissement du vaisseau si scal_p < 0
		norm1 = (unsigned long)(dir2get.x * dir2get.x + dir2get.y * dir2get.y + dir2get.z * dir2get.z); // Norme de dir2get au carr

		if (dir2get.x != 0 || dir2get.z != 0)
			{
			/* Result est en fait le rsultat d'un produit scalaire entre le vecteur xProd
			et un vecteur (dir2get.z, 0, -dir2get.x) qui est un vecteur normal au plan vertical contenant dir2get */
			result = dir2get.z * xProd.x - dir2get.x * xProd.z;
			result2 = abs(result);
			}
		else
			{
			result  = xProd.x; // Pour le test de sens de rotation
			result2 = abs(xProd.x) * fastsqrt(norm1); // Result2 vaut xProd.x * norme(dir2get), produit scalaire entre xProd et le vecteur (norme(dir2get),0,0)
			}

		// Rglage du pas de l'asservissement pour ne pas dpasser la position voulue
		if      (result2 >= (25 * norm1) || scal_p <= 0) step_p = 8;// scal_p Exclut la stabilisation en direction oppose
		else if (result2 >= (3  * norm1)) step_p = 1;
		else		step_p = 0;	// Vaisseau align en pitch

							char buffer[60];
							if (_keytest(RR_CLEAR))
								{
								sprintf(buffer, "%ld %ld", xProd.x, xProd.z); DrawStr(20,20,buffer, A_REPLACE);
								sprintf(buffer, "%ld %ld %ld", dir2get.x, dir2get.y, dir2get.z); DrawStr(20,28,buffer, A_REPLACE);
								sprintf(buffer, "%d %d %d", dir.x, dir.y, dir.z); DrawStr(20,36,buffer, A_REPLACE);
								sprintf(buffer, "%ld %ld %lu %lu ", scal_p, result, result2, norm2); DrawStr(20,44,buffer, A_REPLACE);
								sprintf(buffer, "%d", step_p); DrawStr(20,52,buffer, A_REPLACE);
								DrawVirtual();
								while(_keytest(RR_ENTER));
								while(!_keytest(RR_ENTER));
								}

		if      (result >= 0) A_INC(IAshipC->ang.x, step_p)
		else if (result <  0) A_DEC(IAshipC->ang.x, step_p)
		}

	#ifdef IA_SIMU
	else // Mode Simulation
		{
		// P sera la projection de IAdest dans le plan V,xV,yV, les coordonnes tant exprims dans le repre vaisseau

		// Asservissement en roll
		lP.x = SCAL(dir2get,xV);
		lP.y = SCAL(dir2get,yV);
		lP.z = SCAL(dir2get,zV);

		result2 = lP.x * lP.x;
		result3 = lP.y * lP.y;
		norm1 = result2 + result3; // Norme au carr du vecteur VP

		if 			(result2 >= (625 * norm1>>8)>>8 ) step_r = 8;
		else if (result2 >= (9   * norm1>>8)>>8 ) step_r = 1;
		else		step_r = 0; // calculer le step en roll du vaisseau en fonction de l'impotance de l'angle d'erreur

							/*char buffer[60];
							if (_keytest(RR_CLEAR))
								{
								sprintf(buffer, "%ld %ld %ld", lP.x, lP.y, lP.z); DrawStr(20,20,buffer, A_REPLACE);
								sprintf(buffer, "%ld %ld %ld", dir2get.x, dir2get.y, dir2get.z); DrawStr(20,28,buffer, A_REPLACE);
								sprintf(buffer, "%lu %lu ", result2, norm1); DrawStr(20,36,buffer, A_REPLACE);
								DrawVirtual();
								while(!_keytest(RR_ENTER));
								}*/

		if 			(lP.x >= 0) A_INC(IAshipC->ang.z, step_r)
		else if (lP.x <  0) A_DEC(IAshipC->ang.z, step_r) // Modifier le roll du vaisseau (roulis)

		// Asservissement en pitch dans le repre vaisseau,  condition que le roll soit pas trop mauvais
		if (step_r <= 1)
			{
			norm2 = result3 + lP.z * lP.z;

			if 			(result3 >= (625 * norm2>>8)>>8 || lP.z > 0) spe_step = 5; // Special step : 5 car 256/2^5 = 8
			else if (result3 >= (9   * norm2>>8)>>8 )						 spe_step = 8; // 8 car 256/2^8 = 1
			else 		spe_step = 9; // le spe_step correspond  un nombre de dcalages vers la droite de cosinus ou sinus

			inverse = (IAshipC->ang.x < 128 && IAshipC->ang.x < 384 ? -1 : 1); // Nombre valant -1 ou 1

			if (lP.y >= 0)
				{
				A_INC(IAshipC->ang.x, cos_f(IAshipC->ang.z) >> spe_step)
				A_DEC(IAshipC->ang.y, inverse*(sin_f(IAshipC->ang.z) >> spe_step))
				}
			else if (lP.y < 0)
				{
				A_DEC(IAshipC->ang.x, cos_f(IAshipC->ang.z) >> spe_step)
				A_INC(IAshipC->ang.y, inverse*(sin_f(IAshipC->ang.z) >> spe_step))
				}
			}
		} // Fin de l'asservissement des angles en mode Simulation
	#endif

 // 3.2. Asservissement en vitesse
	dir2get.x = IAdest.x - IAshipC->pos.x;
	dir2get.y = IAdest.y - IAshipC->pos.y;
	dir2get.z = IAdest.z - IAshipC->pos.z; // Vecteur la position du vaisseau ennemi  la mienne
	norm2 = dir2get.x * dir2get.x + dir2get.y * dir2get.y + dir2get.z * dir2get.z; // Nouveau calcul de norm2 avec dir2get a sa vritable taille

	/* Distance d'arrt pour un freinage max  la vitesse v0 :
		Par un petit calcul on trouve d = v0/(2*a) ou a est la valleur absolue de la dclration (ici 4) */

	unsigned short actualSpeed2 = IAshipC->speed * IAshipC->speed; // Vitesse actuelle au carr
	unsigned short accSpeed2 = (IAshipC->speed + 4) * (IAshipC->speed + 4); // Vitesse en cas d'acclration, au carr
	unsigned long distMin = (actualSpeed2>>3) + (IAmode == PURSUIT ? pursuitDist : 0); // Distance minimale de freinage  la vitesse actuelle
	unsigned long accDistMin = (accSpeed2>>3) + (IAmode == PURSUIT ? pursuitDist : 0); // Distance minimale de freinage  la vitesse suprieure

// Le test sur les steps permet d'viter au vaissseau de tourner sur orbite autour du point de destination, ainsi -> freinage et chute  l'intrieur de l'orbite

						/*char buffer[60];
						sprintf(buffer, "%d %d", step_y, step_p); DrawStr(50,10,buffer, A_REPLACE);
						DrawVirtual();*/

	if (IAmode != ESCAPE && IAshipC->speed >= 4 && (norm2 <= distMin * distMin || (controlMode == ARCADE ? step_p+step_y > 2 : step_r > 1))) IAshipC->speed -= 4; // Rallentissement
	else if	(IAshipC->speed < 128 && IAmode != REPAIRnSHOOT && (IAmode == ESCAPE || norm2 > (accDistMin * accDistMin))) IAshipC->speed += 4; // Acclration sans risque de dpasser la position voulue  l'arrt

	unsigned long range = MISS_RANGE * ((unsigned long)(MISS_SPEED + IAshipC->speed));

	// Si aucune des coordonnes n'est suprieure  la porte (sinon c'est dja pas la peine)
	if (!(abs(dir2get.x) > range || abs(dir2get.y) > range || abs(dir2get.z) > range))
		{
		range = range * range;
		if (norm2 < range) range_ok = TRUE; // Possibilit de tirer des missiles si en plus l'alignement est bon
		}

	Vector speedV;
	speedV.x = 0;
	speedV.y = 0;
	speedV.z = -IAshipC->speed;	// Vitesse vers l'avant

	rotations_py(&speedV, &(IAshipC->ang)); // Donner la bonne direction au vecteur vitesse, celle du vaisseau

	// Dplacer le vaisseau dans l'espace suivant son vecteur vitesse
	IAshipC->pos.x += speedV.x;
	IAshipC->pos.y += speedV.y;
	IAshipC->pos.z += speedV.z;


	// Le vaisseau tire si les deux angles d'erreur sont infrieurs ou gaux  1 u.a. (unit d'angle)
	if ((IAmode == PURSUIT || IAmode == REPAIRnSHOOT) && range_ok && (controlMode == ARCADE ? (step_y+step_p <=2) : (step_r + 9-spe_step <=2)))
				IAstatus->missLaunched = TRUE;
	else	IAstatus->missLaunched = FALSE;

	norm2 =  ((long)IAshipC->pos.x)*IAshipC->pos.x
					+((long)IAshipC->pos.y)*IAshipC->pos.y
					+((long)IAshipC->pos.z)*IAshipC->pos.z;
	if (norm2 >= 625000000) // 25000
		IAstatus->explStarted = TRUE; // Le vaisseau part trop loin : desertion -> explosion
		// explStarted est utilis comme message temporaire, l'explosion ne sera active que quand start_explosion sera appelle dans game.c
}